@@ -3,7 +3,6 @@
 
 use bytes::Bytes;
 use log::debug;
-use pretty_hex::PrettyHex;
 use sdp_types::Media;
 use std::{net::IpAddr, num::NonZeroU16};
 use url::Url;
@@ -262,44 +261,60 @@ fn parse_media(base_url: &Url, media_description: &Media) -> Result<Stream, Stri
     let mut rtpmap = None;
     let mut fmtp = None;
     let mut control = None;
+    let mut framerate = None;
     for a in &media_description.attributes {
-        if a.attribute == "rtpmap" {
-            let v = a
-                .value
-                .as_ref()
-                .ok_or_else(|| "rtpmap attribute with no value".to_string())?;
-            // https://tools.ietf.org/html/rfc8866#section-6.6
-            // rtpmap-value = payload-type SP encoding-name
-            //   "/" clock-rate [ "/" encoding-params ]
-            // payload-type = zero-based-integer
-            // encoding-name = token
-            // clock-rate = integer
-            // encoding-params = channels
-            // channels = integer
-            let (rtpmap_payload_type, v) = v
-                .split_once(' ')
-                .ok_or_else(|| "invalid rtmap attribute".to_string())?;
-            if rtpmap_payload_type == rtp_payload_type_str {
-                rtpmap = Some(v);
+        match a.attribute.as_str() {
+            "rtpmap" => {
+                let v = a
+                    .value
+                    .as_ref()
+                    .ok_or_else(|| "rtpmap attribute with no value".to_string())?
+                    .trim_end_matches(' ');
+                // https://tools.ietf.org/html/rfc8866#section-6.6
+                // rtpmap-value = payload-type SP encoding-name
+                //   "/" clock-rate [ "/" encoding-params ]
+                // payload-type = zero-based-integer
+                // encoding-name = token
+                // clock-rate = integer
+                // encoding-params = channels
+                // channels = integer
+                //
+                // At least one camera (improperly) sends a trailing space; trim this.
+                let (rtpmap_payload_type, v) = v
+                    .split_once(' ')
+                    .ok_or_else(|| "invalid rtmap attribute".to_string())?;
+                if rtpmap_payload_type == rtp_payload_type_str {
+                    rtpmap = Some(v);
+                }
             }
-        } else if a.attribute == "fmtp" {
-            // Similarly starts with payload-type SP.
-            let v = a
-                .value
-                .as_ref()
-                .ok_or_else(|| "fmtp attribute with no value".to_string())?;
-            let (fmtp_payload_type, v) = v
-                .split_once(' ')
-                .ok_or_else(|| "invalid fmtp attribute".to_string())?;
-            if fmtp_payload_type == rtp_payload_type_str {
-                fmtp = Some(v);
+            "fmtp" => {
+                // Similarly starts with payload-type SP.
+                let v = a
+                    .value
+                    .as_ref()
+                    .ok_or_else(|| "fmtp attribute with no value".to_string())?;
+                let (fmtp_payload_type, v) = v
+                    .split_once(' ')
+                    .ok_or_else(|| "invalid fmtp attribute".to_string())?;
+                if fmtp_payload_type == rtp_payload_type_str {
+                    fmtp = Some(v);
+                }
             }
-        } else if a.attribute == "control" {
-            control = a
-                .value
-                .as_deref()
-                .map(|c| join_control(base_url, c))
-                .transpose()?;
+            "control" => {
+                control = a
+                    .value
+                    .as_deref()
+                    .map(|c| join_control(base_url, c))
+                    .transpose()?;
+            }
+            "framerate" => {
+                if let Some(s) = a.value.as_ref() {
+                    if let Ok(f) = s.parse::<f32>() {
+                        framerate = Some(f);
+                    }
+                }
+            }
+            _ => (),
         }
     }
 
@@ -362,10 +377,36 @@ fn parse_media(base_url: &Url, media_description: &Media) -> Result<Stream, Stri
         control,
         sockets: None,
         channels,
+        framerate,
         state: super::StreamState::Uninit,
     })
 }
 
+/// Holds something that's supposed to be mostly ASCII.
+///
+/// Some SDP values are allowed to be UTF-8-encoded ISO-10646 characters, but mostly it's
+/// supposed to be valid ASCII. This has a `Debug` impl that is fairly readable for ASCII
+/// characters and can be copy'n'pasted into Python's `codecs.decode(..., 'string_escape'`)`
+/// or the like.
+struct MostlyAscii<'a>(&'a [u8]);
+
+impl<'a> std::fmt::Debug for MostlyAscii<'a> {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "\"")?;
+        for &b in self.0 {
+            match b {
+                b'"' => write!(f, "\"")?,
+                b'\r' => write!(f, "\\r")?,
+                // Note this writes newlines in unescaped form.
+                b' ' | b'\n' | 0x20..=0x7E => write!(f, "{}", b as char)?,
+                _ => write!(f, "\\x{:02X}", b)?,
+            }
+        }
+        write!(f, "\"")?;
+        Ok(())
+    }
+}
+
 /// Parses a successful RTSP `DESCRIBE` response into a [Presentation].
 /// On error, returns a string which is expected to be packed into an `RtspProtocolError`.
 pub(crate) fn parse_describe(
@@ -380,13 +421,9 @@ pub(crate) fn parse_describe(
         ));
     }
 
-    let sdp = sdp_types::Session::parse(&response.body()[..]).map_err(|e| {
-        format!(
-            "Unable to parse SDP: {}\n\n{:#?}",
-            e,
-            response.body().hex_dump()
-        )
-    })?;
+    let raw_sdp = MostlyAscii(&response.body()[..]);
+    let sdp = sdp_types::Session::parse(raw_sdp.0)
+        .map_err(|e| format!("Unable to parse SDP: {}\n\n{:#?}", e, raw_sdp,))?;
 
     // https://tools.ietf.org/html/rfc2326#appendix-C.1.1
     let base_url = response
@@ -398,7 +435,7 @@ pub(crate) fn parse_describe(
                 .map(|v| (rtsp_types::headers::CONTENT_LOCATION, v))
         })
         .map(|(h, v)| Url::parse(v.as_str()).map_err(|e| format!("bad {} {:?}: {}", h, v, e)))
-        .unwrap_or(Ok(request_url.clone()))?;
+        .unwrap_or_else(|| Ok(request_url.clone()))?;
 
     let mut control = None;
     let mut tool = None;
@@ -411,7 +448,7 @@ pub(crate) fn parse_describe(
                 .transpose()?;
             break;
         } else if a.attribute == "tool" {
-            tool = a.value.as_deref().map(Into::into);
+            tool = a.value.as_deref().map(super::Tool::new);
         }
     }
     let control = control.unwrap_or(request_url);
@@ -421,8 +458,12 @@ pub(crate) fn parse_describe(
         .iter()
         .enumerate()
         .map(|(i, m)| {
-            parse_media(&base_url, &m)
-                .map_err(|e| format!("Unable to parse stream {}: {}\n\n{:#?}", i, &e, &m))
+            parse_media(&base_url, m).map_err(|e| {
+                format!(
+                    "Unable to parse stream {}: {}\nraw SDP: {:#?}",
+                    i, &e, raw_sdp
+                )
+            })
         })
         .collect::<Result<Vec<Stream>, String>>()?;
 
@@ -552,8 +593,7 @@ pub(crate) fn parse_play(
             .strip_prefix("url=")
             .ok_or_else(|| "RTP-Info missing stream URL".to_string())?;
         let url = join_control(&presentation.base_url, url)?;
-        let stream;
-        if presentation.streams.len() == 1 {
+        let stream = if presentation.streams.len() == 1 {
             // The server is allowed to not specify a stream control URL for
             // single-stream presentations. Additionally, some buggy
             // cameras (eg the GW Security GW4089IP) use an incorrect URL.
@@ -562,13 +602,13 @@ pub(crate) fn parse_play(
             // servers to be forgiving of clients with single-stream
             // containers.
             // https://datatracker.ietf.org/doc/html/rfc2326#section-14.3
-            stream = Some(&mut presentation.streams[0]);
+            Some(&mut presentation.streams[0])
         } else {
-            stream = presentation
+            presentation
                 .streams
                 .iter_mut()
-                .find(|s| matches!(&s.control, Some(u) if u == &url));
-        }
+                .find(|s| matches!(&s.control, Some(u) if u == &url))
+        };
         let stream = match stream {
             Some(s) => s,
             None => {
@@ -1291,4 +1331,31 @@ mod tests {
             _ => panic!(),
         };
     }
+
+    /// Tests parsing SDP from `the macro-video rtsp server`, with missing origin line.
+    #[test]
+    fn macrovideo() {
+        // DESCRIBE.
+        let p = parse_describe(
+            "rtsp://camera",
+            include_bytes!("testdata/macrovideo_describe.txt"),
+        )
+        .unwrap();
+        assert_eq!(p.streams.len(), 1);
+    }
+
+    /// Tests parsing SDP from an "IPCAM C9F0SeZ3N0PbL0" [sic], as quoted in
+    /// [imoonfire-nvr#213](https://github.com/scottlamb/moonfire-nvr/issues/213).
+    ///
+    /// This camera notably sends a trailing space in its `rtpmap` attribute.
+    #[test]
+    fn ipcam() {
+        // DESCRIBE.
+        let p = parse_describe(
+            "rtsp://camera",
+            include_bytes!("testdata/ipcam_describe.txt"),
+        )
+        .unwrap();
+        assert_eq!(p.streams.len(), 1);
+    }
 }
