@@ -5,7 +5,6 @@
 
 use bytes::{Buf, Bytes};
 use log::{debug, trace};
-use pretty_hex::PrettyHex;
 
 use crate::client::PacketItem;
 use crate::{ConnectionContext, Error, ErrorInt, PacketContext};
@@ -44,7 +43,7 @@ impl std::fmt::Debug for Packet {
             .field("sequence_number", &self.sequence_number)
             .field("loss", &self.loss)
             .field("mark", &self.mark)
-            .field("payload", &self.payload.hex_dump())
+            .field("payload", &crate::hex::LimitedHex::new(&self.payload, 64))
             .finish()
     }
 }
@@ -81,20 +80,32 @@ pub struct SenderReport {
 pub struct InorderParser {
     ssrc: Option<u32>,
     next_seq: Option<u16>,
+
+    /// True iff `ssrc` was set from the beginning via a `RTP-Info` header.
+    initial_ssrc: bool,
+
+    /// Total packets seen in this stream.
+    seen_packets: u64,
 }
 
 impl InorderParser {
     pub fn new(ssrc: Option<u32>, next_seq: Option<u16>) -> Self {
-        Self { ssrc, next_seq }
+        Self {
+            ssrc,
+            next_seq,
+            initial_ssrc: ssrc.is_some(),
+            seen_packets: 0,
+        }
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub fn rtp(
         &mut self,
         session_options: &SessionOptions,
+        tool: Option<&super::Tool>,
         conn_ctx: &ConnectionContext,
         pkt_ctx: &PacketContext,
         timeline: &mut Timeline,
-        runtime_handle: Option<&tokio::runtime::Handle>,
         stream_id: usize,
         mut data: Bytes,
     ) -> Result<Option<PacketItem>, Error> {
@@ -107,7 +118,7 @@ impl InorderParser {
                     "corrupt RTP header while expecting seq={:04x?}: {:?}\n{:#?}",
                     &self.next_seq,
                     e,
-                    data.hex_dump(),
+                    crate::hex::LimitedHex::new(&data, 64),
                 ),
             })
         })?;
@@ -126,12 +137,9 @@ impl InorderParser {
         let sequence_number = u16::from_be_bytes([data[2], data[3]]); // I don't like rtsp_rs::Seq.
         let ssrc = reader.ssrc();
         let loss = sequence_number.wrapping_sub(self.next_seq.unwrap_or(sequence_number));
-        let is_tcp = matches!(session_options.transport, super::Transport::Tcp);
         if matches!(self.ssrc, Some(s) if s != ssrc) {
-            if let (Some(session_group), true) = (session_options.session_group.as_ref(), is_tcp) {
-                session_group
-                    .clone()
-                    .note_stale_live555_data(runtime_handle.cloned());
+            if matches!(session_options.transport, super::Transport::Tcp) {
+                super::note_stale_live555_data(tool, session_options);
             }
             bail!(ErrorInt::RtpPacketError {
                 conn_ctx: *conn_ctx,
@@ -140,8 +148,9 @@ impl InorderParser {
                 ssrc,
                 sequence_number,
                 description: format!(
-                    "Wrong ssrc; expecting ssrc={:08x?} seq={:04x?}",
-                    self.ssrc, self.next_seq
+                    "Wrong ssrc after {} packets; expecting ssrc={:08x?} seq={:04x?} \
+                     (initial ssrc: {:?})",
+                    self.seen_packets, self.ssrc, self.next_seq, self.initial_ssrc,
                 ),
             });
         }
@@ -194,6 +203,7 @@ impl InorderParser {
         data.truncate(payload_range.end);
         data.advance(payload_range.start);
         self.next_seq = Some(sequence_number.wrapping_add(1));
+        self.seen_packets += 1;
         Ok(Some(PacketItem::RtpPacket(Packet {
             ctx: *pkt_ctx,
             stream_id,
@@ -206,12 +216,13 @@ impl InorderParser {
         })))
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub fn rtcp(
         &mut self,
         session_options: &SessionOptions,
+        tool: Option<&super::Tool>,
         pkt_ctx: &PacketContext,
         timeline: &mut Timeline,
-        runtime_handle: Option<&tokio::runtime::Handle>,
         stream_id: usize,
         data: Bytes,
     ) -> Result<Option<PacketItem>, String> {
@@ -236,14 +247,8 @@ impl InorderParser {
 
                     let ssrc = pkt.ssrc();
                     if matches!(self.ssrc, Some(s) if s != ssrc) {
-                        use super::Transport;
-                        if let (Some(session_group), Transport::Tcp) = (
-                            session_options.session_group.as_ref(),
-                            session_options.transport,
-                        ) {
-                            session_group
-                                .clone()
-                                .note_stale_live555_data(runtime_handle.cloned());
+                        if matches!(session_options.transport, super::Transport::Tcp) {
+                            super::note_stale_live555_data(tool, session_options);
                         }
                         return Err(format!(
                             "Expected ssrc={:08x?}, got RTCP SR ssrc={:08x}",
@@ -283,10 +288,10 @@ mod tests {
         // Normal packet.
         match parser.rtp(
             &SessionOptions::default(),
+            None,
             &ConnectionContext::dummy(),
             &PacketContext::dummy(),
             &mut timeline,
-            None,
             0,
             rtp_rs::RtpPacketBuilder::new()
                 .payload_type(105)
@@ -306,10 +311,10 @@ mod tests {
         // Mystery pt=50 packet with same sequence number.
         match parser.rtp(
             &SessionOptions::default(),
+            None,
             &ConnectionContext::dummy(),
             &PacketContext::dummy(),
             &mut timeline,
-            None,
             0,
             rtp_rs::RtpPacketBuilder::new()
                 .payload_type(50)
@@ -335,10 +340,10 @@ mod tests {
         let session_options = SessionOptions::default().transport(crate::client::Transport::Udp);
         match parser.rtp(
             &session_options,
+            None,
             &ConnectionContext::dummy(),
             &PacketContext::dummy(),
             &mut timeline,
-            None,
             0,
             rtp_rs::RtpPacketBuilder::new()
                 .payload_type(96)
@@ -359,10 +364,10 @@ mod tests {
 
         match parser.rtp(
             &session_options,
+            None,
             &ConnectionContext::dummy(),
             &PacketContext::dummy(),
             &mut timeline,
-            None,
             0,
             rtp_rs::RtpPacketBuilder::new()
                 .payload_type(96)
@@ -381,10 +386,10 @@ mod tests {
 
         match parser.rtp(
             &session_options,
+            None,
             &ConnectionContext::dummy(),
             &PacketContext::dummy(),
             &mut timeline,
-            None,
             0,
             rtp_rs::RtpPacketBuilder::new()
                 .payload_type(96)
