@@ -50,6 +50,8 @@ struct ChannelConfig {
     ncc: u16,
 
     /// A human-friendly name for the channel configuration.
+    // The name is used in tests and in the Debug output. Suppress dead code warning.
+    #[cfg_attr(not(test), allow(dead_code))]
     name: &'static str,
 }
 
@@ -68,7 +70,7 @@ const CHANNEL_CONFIGS: [Option<ChannelConfig>; 8] = [
 impl AudioSpecificConfig {
     /// Parses from raw bytes.
     fn parse(raw: &[u8]) -> Result<Self, String> {
-        let mut r = bitreader::BitReader::new(&raw[..]);
+        let mut r = bitreader::BitReader::new(raw);
         let audio_object_type = match r
             .read_u8(5)
             .map_err(|e| format!("unable to read audio_object_type: {}", e))?
@@ -211,23 +213,31 @@ fn set_length(len: usize, data: &mut [u8]) -> Result<usize, String> {
 /// Writes a box length and type (four-character code) for everything appended
 /// in the supplied scope.
 macro_rules! write_box {
-    ($buf:expr, $fourcc:expr, $b:block) => {{
-        let _: &mut BytesMut = $buf; // type-check.
-        let pos_start = $buf.len();
-        let fourcc: &[u8; 4] = $fourcc;
-        $buf.extend_from_slice(&[0, 0, 0, 0, fourcc[0], fourcc[1], fourcc[2], fourcc[3]]);
-        let r = {
-            $b;
-        };
-        let pos_end = $buf.len();
-        let len = pos_end.checked_sub(pos_start).unwrap();
-        $buf[pos_start..pos_start + 4].copy_from_slice(
-            &u32::try_from(len)
-                .map_err(|_| format!("box length {} exceeds u32::MAX", len))?
-                .to_be_bytes()[..],
-        );
-        r
-    }};
+    ($buf:expr, $fourcc:expr, $b:block) => {
+        // The caller uses `&mut buf`. clippy likes to complain about the `&mut`
+        // being unnecessary for len(), but it is necessary for other things.
+        // The macro also can't store `$buf` in its own local, because `$b`
+        // is expected to reference `$buf` via the original name.
+        #[allow(clippy::unnecessary_mut_passed)]
+        {
+            let _: &mut BytesMut = $buf; // type-check.
+
+            let pos_start = $buf.len();
+            let fourcc: &[u8; 4] = $fourcc;
+            $buf.extend_from_slice(&[0, 0, 0, 0, fourcc[0], fourcc[1], fourcc[2], fourcc[3]]);
+            let r = {
+                $b;
+            };
+            let pos_end = $buf.len();
+            let len = pos_end.checked_sub(pos_start).unwrap();
+            $buf[pos_start..pos_start + 4].copy_from_slice(
+                &u32::try_from(len)
+                    .map_err(|_| format!("box length {} exceeds u32::MAX", len))?
+                    .to_be_bytes()[..],
+            );
+            r
+        }
+    };
 }
 
 /// Writes a descriptor tag and length for everything appended in the supplied
@@ -425,6 +435,11 @@ pub(crate) struct Depacketizer {
     state: DepacketizerState,
 }
 
+/// [DepacketizerState] holding access units within a single RTP packet.
+///
+/// This is the state used when there are multiple access units within a packet
+/// (thus the name), when there's a single access unit, and even at the
+/// beginning of a fragment.
 #[derive(Debug)]
 struct Aggregate {
     ctx: crate::PacketContext,
@@ -450,10 +465,11 @@ struct Aggregate {
     /// The buffer, positioned at frame 0's header.
     buf: Bytes,
 
-    /// The index in range `[0, frame_count)` of the next frame to output.
+    /// The index in range `[0, frame_count)` of the next frame to return from `pull`.
     frame_i: u16,
 
-    /// The non-zero total frames within this aggregate.
+    /// The total non-zero total frames within this aggregate (including ones which have already
+    /// been returned by `pull`).
     frame_count: u16,
 
     /// The starting byte offset of `frame_i`'s data within `buf`.
@@ -465,6 +481,7 @@ struct Aggregate {
     mark: bool,
 }
 
+/// The received prefix of a single access unit which has been spread across multiple packets.
 #[derive(Debug)]
 struct Fragment {
     rtp_timestamp: u16,
@@ -481,15 +498,35 @@ struct Fragment {
     buf: BytesMut,
 }
 
+/// State of the depacketizer between calls to `push` and `pull`.
 #[derive(Debug)]
 #[allow(clippy::large_enum_variant)]
 enum DepacketizerState {
-    Idle { prev_loss: u16 },
+    /// State when there's no buffered data.
+    Idle {
+        prev_loss: u16,
+        loss_since_mark: bool,
+    },
+
+    /// State after a packet has been RTP packet has been received. As described at
+    /// [`Aggregate`], this may hold the first packet of a fragment, one packet, or multiple
+    /// complete packets.
     Aggregated(Aggregate),
+
+    /// State when a prefix of a fragmented packet has been received.
     Fragmented(Fragment),
     Ready(super::AudioFrame),
 }
 
+impl Default for DepacketizerState {
+    fn default() -> Self {
+        DepacketizerState::Idle {
+            prev_loss: 0,
+            loss_since_mark: false,
+        }
+    }
+}
+
 impl Depacketizer {
     pub(super) fn new(
         clock_rate: u32,
@@ -507,7 +544,7 @@ impl Depacketizer {
         }
         Ok(Self {
             config,
-            state: DepacketizerState::Idle { prev_loss: 0 },
+            state: DepacketizerState::default(),
         })
     }
 
@@ -516,12 +553,17 @@ impl Depacketizer {
     }
 
     pub(super) fn push(&mut self, mut pkt: Packet) -> Result<(), String> {
-        if pkt.loss > 0 && matches!(self.state, DepacketizerState::Fragmented(_)) {
-            log::debug!(
-                "Discarding fragmented AAC frame due to loss of {} RTP packets.",
-                pkt.loss
-            );
-            self.state = DepacketizerState::Idle { prev_loss: 0 };
+        if pkt.loss > 0 {
+            if let DepacketizerState::Fragmented(ref mut f) = self.state {
+                log::debug!(
+                    "Discarding in-progress fragmented AAC frame due to loss of {} RTP packets.",
+                    pkt.loss
+                );
+                self.state = DepacketizerState::Idle {
+                    prev_loss: f.loss, // note this packet's loss will be added in later.
+                    loss_since_mark: true,
+                };
+            }
         }
 
         // Read the AU headers.
@@ -562,9 +604,10 @@ impl Depacketizer {
                 match (frag.buf.len() + data.len()).cmp(&size) {
                     std::cmp::Ordering::Less => {
                         if pkt.mark {
-                            if frag.loss > 0 {
+                            if frag.loss_since_mark {
                                 self.state = DepacketizerState::Idle {
                                     prev_loss: frag.loss,
+                                    loss_since_mark: false,
                                 };
                                 return Ok(());
                             }
@@ -575,6 +618,7 @@ impl Depacketizer {
                                 size
                             ));
                         }
+                        frag.buf.extend_from_slice(data);
                     }
                     std::cmp::Ordering::Equal => {
                         if !pkt.mark {
@@ -583,7 +627,6 @@ impl Depacketizer {
                             );
                         }
                         frag.buf.extend_from_slice(data);
-                        println!("au {}: len-{}, fragmented", &pkt.timestamp, size);
                         self.state = DepacketizerState::Ready(super::AudioFrame {
                             ctx: pkt.ctx,
                             loss: frag.loss,
@@ -597,14 +640,17 @@ impl Depacketizer {
                 }
             }
             DepacketizerState::Aggregated(_) => panic!("push when already in state aggregated"),
-            DepacketizerState::Idle { prev_loss } => {
+            DepacketizerState::Idle {
+                prev_loss,
+                loss_since_mark,
+            } => {
                 if au_headers_count == 0 {
                     return Err("aggregate with no headers".to_string());
                 }
                 self.state = DepacketizerState::Aggregated(Aggregate {
                     ctx: pkt.ctx,
                     loss: *prev_loss + pkt.loss,
-                    loss_since_mark: pkt.loss > 0,
+                    loss_since_mark: *loss_since_mark || pkt.loss > 0,
                     stream_id: pkt.stream_id,
                     ssrc: pkt.ssrc,
                     sequence_number: pkt.sequence_number,
@@ -625,13 +671,13 @@ impl Depacketizer {
         &mut self,
         conn_ctx: &ConnectionContext,
     ) -> Result<Option<super::CodecItem>, Error> {
-        match std::mem::replace(&mut self.state, DepacketizerState::Idle { prev_loss: 0 }) {
+        match std::mem::take(&mut self.state) {
             s @ DepacketizerState::Idle { .. } | s @ DepacketizerState::Fragmented(..) => {
                 self.state = s;
                 Ok(None)
             }
             DepacketizerState::Ready(f) => {
-                self.state = DepacketizerState::Idle { prev_loss: 0 };
+                self.state = DepacketizerState::default();
                 Ok(Some(CodecItem::AudioFrame(f)))
             }
             DepacketizerState::Aggregated(mut agg) => {
@@ -660,6 +706,17 @@ impl Depacketizer {
                         ));
                     }
                     if agg.mark {
+                        if agg.loss_since_mark {
+                            log::debug!(
+                                "Discarding in-progress fragmented AAC frame due to loss of {} RTP packets.",
+                                agg.loss
+                            );
+                            self.state = DepacketizerState::Idle {
+                                prev_loss: agg.loss,
+                                loss_since_mark: false,
+                            };
+                            return Ok(None);
+                        }
                         return Err(error(
                             *conn_ctx,
                             agg,
@@ -734,18 +791,475 @@ fn error(conn_ctx: ConnectionContext, agg: Aggregate, description: String) -> Er
 
 #[cfg(test)]
 mod tests {
+    use super::*;
+
     #[test]
     fn parse_audio_specific_config() {
-        let dahua = super::AudioSpecificConfig::parse(&[0x11, 0x88]).unwrap();
+        let dahua = AudioSpecificConfig::parse(&[0x11, 0x88]).unwrap();
         assert_eq!(dahua.sampling_frequency, 48_000);
         assert_eq!(dahua.channels.name, "mono");
 
-        let bunny = super::AudioSpecificConfig::parse(&[0x14, 0x90]).unwrap();
+        let bunny = AudioSpecificConfig::parse(&[0x14, 0x90]).unwrap();
         assert_eq!(bunny.sampling_frequency, 12_000);
         assert_eq!(bunny.channels.name, "stereo");
 
-        let rfc3640 = super::AudioSpecificConfig::parse(&[0x11, 0xB0]).unwrap();
+        let rfc3640 = AudioSpecificConfig::parse(&[0x11, 0xB0]).unwrap();
         assert_eq!(rfc3640.sampling_frequency, 48_000);
         assert_eq!(rfc3640.channels.name, "5.1");
     }
+
+    #[test]
+    fn depacketize_happy_path() {
+        let mut d = Depacketizer::new(
+            48_000, // clock rate, as specified in rtpmap
+            None, // channels, as specified in rtpmap
+            Some("streamtype=5;profile-level-id=1;mode=AAC-hbr;sizelength=13;indexlength=3;indexdeltalength=3;config=1188"),
+        ).unwrap();
+        let timestamp = crate::Timestamp {
+            timestamp: 42,
+            clock_rate: NonZeroU32::new(48_000).unwrap(),
+            start: 0,
+        };
+
+        // Single frame.
+        d.push(Packet {
+            // single frame.
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 0,
+            mark: true,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x20, // AU-header: AU-size=4 + AU-index=0
+                b'a', b's', b'd', b'f',
+            ]),
+        })
+        .unwrap();
+        let a = match d.pull(&ConnectionContext::dummy()).unwrap() {
+            Some(CodecItem::AudioFrame(a)) => a,
+            _ => unreachable!(),
+        };
+        assert_eq!(a.timestamp, timestamp);
+        assert_eq!(&a.data[..], b"asdf");
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+
+        // Aggregate of 3 frames.
+        d.push(Packet {
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 0,
+            mark: true,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x30, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 3 headers
+                0x00, 0x18, // AU-header: AU-size=3 + AU-index=0
+                0x00, 0x18, // AU-header: AU-size=3 + AU-index-delta=0
+                0x00, 0x18, // AU-header: AU-size=3 + AU-index-delta=0
+                b'f', b'o', b'o', b'b', b'a', b'r', b'b', b'a', b'z',
+            ]),
+        })
+        .unwrap();
+        let a = match d.pull(&ConnectionContext::dummy()).unwrap() {
+            Some(CodecItem::AudioFrame(a)) => a,
+            _ => unreachable!(),
+        };
+        assert_eq!(a.timestamp, timestamp);
+        assert_eq!(&a.data[..], b"foo");
+        let a = match d.pull(&ConnectionContext::dummy()).unwrap() {
+            Some(CodecItem::AudioFrame(a)) => a,
+            _ => unreachable!(),
+        };
+        assert_eq!(a.timestamp, timestamp.try_add(1_024).unwrap());
+        assert_eq!(&a.data[..], b"bar");
+        let a = match d.pull(&ConnectionContext::dummy()).unwrap() {
+            Some(CodecItem::AudioFrame(a)) => a,
+            _ => unreachable!(),
+        };
+        assert_eq!(a.timestamp, timestamp.try_add(2_048).unwrap());
+        assert_eq!(&a.data[..], b"baz");
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+
+        // Fragment across 3 packets.
+        d.push(Packet {
+            // fragment 1/3.
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 0,
+            mark: false,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x48, // AU-header: AU-size=9 + AU-index=0
+                b'f', b'o', b'o',
+            ]),
+        })
+        .unwrap();
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+        d.push(Packet {
+            // fragment 2/3.
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 0,
+            mark: false,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x48, // AU-header: AU-size=9 + AU-index=0
+                b'b', b'a', b'r',
+            ]),
+        })
+        .unwrap();
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+        d.push(Packet {
+            // fragment 3/3.
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 0,
+            mark: true,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x48, // AU-header: AU-size=9 + AU-index=0
+                b'b', b'a', b'z',
+            ]),
+        })
+        .unwrap();
+        let a = match d.pull(&ConnectionContext::dummy()).unwrap() {
+            Some(CodecItem::AudioFrame(a)) => a,
+            _ => unreachable!(),
+        };
+        assert_eq!(a.timestamp, timestamp);
+        assert_eq!(&a.data[..], b"foobarbaz");
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+    }
+
+    /// Tests that depacketization skips/reports a frame in which its first packet was lost.
+    #[test]
+    fn depacketize_fragment_initial_loss() {
+        let mut d = Depacketizer::new(
+            48_000, // clock rate, as specified in rtpmap
+            None, // channels, as specified in rtpmap
+            Some("streamtype=5;profile-level-id=1;mode=AAC-hbr;sizelength=13;indexlength=3;indexdeltalength=3;config=1188"),
+        ).unwrap();
+        let timestamp = crate::Timestamp {
+            timestamp: 42,
+            clock_rate: NonZeroU32::new(48_000).unwrap(),
+            start: 0,
+        };
+
+        // Fragment
+        d.push(Packet {
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 1,
+            mark: false,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x48, // AU-header: AU-size=9 + AU-index=0
+                b'b', b'a', b'r',
+            ]),
+        })
+        .unwrap();
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+        d.push(Packet {
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 0,
+            mark: true,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x48, // AU-header: AU-size=9 + AU-index=0
+                b'b', b'a', b'z',
+            ]),
+        })
+        .unwrap();
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+
+        // Following frame reports the loss.
+        d.push(Packet {
+            // single frame.
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 0,
+            mark: true,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x20, // AU-header: AU-size=4 + AU-index=0
+                b'a', b's', b'd', b'f',
+            ]),
+        })
+        .unwrap();
+        let a = match d.pull(&ConnectionContext::dummy()).unwrap() {
+            Some(CodecItem::AudioFrame(a)) => a,
+            _ => unreachable!(),
+        };
+        assert_eq!(a.loss, 1);
+        assert_eq!(&a.data[..], b"asdf");
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+    }
+
+    /// Tests that depacketization skips/reports a frame in which an interior frame is lost.
+    #[test]
+    fn depacketize_fragment_interior_loss() {
+        let mut d = Depacketizer::new(
+            48_000, // clock rate, as specified in rtpmap
+            None, // channels, as specified in rtpmap
+            Some("streamtype=5;profile-level-id=1;mode=AAC-hbr;sizelength=13;indexlength=3;indexdeltalength=3;config=1188"),
+        ).unwrap();
+        let timestamp = crate::Timestamp {
+            timestamp: 42,
+            clock_rate: NonZeroU32::new(48_000).unwrap(),
+            start: 0,
+        };
+
+        d.push(Packet {
+            // 1/3
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 0,
+            mark: false,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x48, // AU-header: AU-size=9 + AU-index=0
+                b'f', b'o', b'o',
+            ]),
+        })
+        .unwrap();
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+        // Fragment 2/3 is lost
+        d.push(Packet {
+            // 3/3 reports the loss
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 1,
+            mark: true,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x48, // AU-header: AU-size=9 + AU-index=0
+                b'b', b'a', b'z',
+            ]),
+        })
+        .unwrap();
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+
+        // Following frame reports the loss.
+        d.push(Packet {
+            // single frame.
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 0,
+            mark: true,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x20, // AU-header: AU-size=4 + AU-index=0
+                b'a', b's', b'd', b'f',
+            ]),
+        })
+        .unwrap();
+        let a = match d.pull(&ConnectionContext::dummy()).unwrap() {
+            Some(CodecItem::AudioFrame(a)) => a,
+            _ => unreachable!(),
+        };
+        assert_eq!(a.loss, 1);
+        assert_eq!(&a.data[..], b"asdf");
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+    }
+
+    /// Tests that depacketization skips/reports a frame in which the interior frame is lost.
+    #[test]
+    fn depacketize_fragment_final_loss() {
+        let mut d = Depacketizer::new(
+            48_000, // clock rate, as specified in rtpmap
+            None, // channels, as specified in rtpmap
+            Some("streamtype=5;profile-level-id=1;mode=AAC-hbr;sizelength=13;indexlength=3;indexdeltalength=3;config=1188"),
+        ).unwrap();
+        let timestamp = crate::Timestamp {
+            timestamp: 42,
+            clock_rate: NonZeroU32::new(48_000).unwrap(),
+            start: 0,
+        };
+
+        d.push(Packet {
+            // 1/3
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 0,
+            mark: false,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x48, // AU-header: AU-size=9 + AU-index=0
+                b'f', b'o', b'o',
+            ]),
+        })
+        .unwrap();
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+        // Fragment 2/3 is lost
+        d.push(Packet {
+            // 3/3 reports the loss
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 1,
+            mark: true,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x48, // AU-header: AU-size=9 + AU-index=0
+                b'b', b'a', b'z',
+            ]),
+        })
+        .unwrap();
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+
+        // Following frame reports the loss.
+        d.push(Packet {
+            // single frame.
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 0,
+            mark: true,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x20, // AU-header: AU-size=4 + AU-index=0
+                b'a', b's', b'd', b'f',
+            ]),
+        })
+        .unwrap();
+        let a = match d.pull(&ConnectionContext::dummy()).unwrap() {
+            Some(CodecItem::AudioFrame(a)) => a,
+            _ => unreachable!(),
+        };
+        assert_eq!(a.loss, 1);
+        assert_eq!(&a.data[..], b"asdf");
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+    }
+
+    /// Tests the distinction between `loss` and `loss_since_last_mark`.
+    ///
+    /// Old loss should get reported via the CodecItem but shouldn't suppress
+    /// error checking.
+    #[test]
+    fn depacketize_fragment_old_loss_doesnt_prevent_error() {
+        let mut d = Depacketizer::new(
+            48_000, // clock rate, as specified in rtpmap
+            None, // channels, as specified in rtpmap
+            Some("streamtype=5;profile-level-id=1;mode=AAC-hbr;sizelength=13;indexlength=3;indexdeltalength=3;config=1188"),
+        ).unwrap();
+        let timestamp = crate::Timestamp {
+            timestamp: 42,
+            clock_rate: NonZeroU32::new(48_000).unwrap(),
+            start: 0,
+        };
+
+        d.push(Packet {
+            // end of previous fragment, first parts missing.
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 1,
+            mark: true,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x48, // AU-header: AU-size=9 + AU-index=0
+                b'b', b'a', b'r',
+            ]),
+        })
+        .unwrap();
+        assert!(d.pull(&ConnectionContext::dummy()).unwrap().is_none());
+
+        // Incomplete fragment with no reported loss.
+        d.push(Packet {
+            // end of previous fragment, first parts missing.
+            ctx: crate::PacketContext::dummy(),
+            stream_id: 0,
+            timestamp,
+            ssrc: 0,
+            sequence_number: 0,
+            loss: 0,
+            mark: true,
+            payload: Bytes::from_static(&[
+                // https://datatracker.ietf.org/doc/html/rfc3640#section-3.2.1
+                0x00,
+                0x10, // AU-headers-length: 16 bits (13-bit size + 3-bit index) => 1 header
+                0x00, 0x48, // AU-header: AU-size=9 + AU-index=0
+                b'b', b'a', b'r',
+            ]),
+        })
+        .unwrap();
+        let e = d.pull(&ConnectionContext::dummy()).unwrap_err();
+        let e_str = e.to_string();
+        assert!(
+            e_str.contains("mark can't be set on beginning of fragment"),
+            "{}",
+            e
+        );
+    }
 }
