@@ -6,6 +6,10 @@
 //! Currently this is useful for clients; it will be extended to support
 //! servers and proxies.
 
+#![forbid(clippy::print_stderr, clippy::print_stdout)]
+// I prefer to use from_str_radix(..., 10) to explicitly note the base.
+#![allow(clippy::from_str_radix_10)]
+
 use bytes::Bytes;
 use log::trace;
 //use failure::{bail, format_err, Error};
@@ -20,6 +24,7 @@ use std::ops::Range;
 mod error;
 mod rtcp;
 
+mod hex;
 #[cfg(test)]
 mod testutil;
 
@@ -269,7 +274,6 @@ pub struct ConnectionContext {
     local_addr: std::net::SocketAddr,
     peer_addr: std::net::SocketAddr,
     established_wall: WallTime,
-    established: std::time::Instant,
 }
 
 impl ConnectionContext {
@@ -280,7 +284,6 @@ impl ConnectionContext {
             local_addr: addr,
             peer_addr: addr,
             established_wall: WallTime::now(),
-            established: std::time::Instant::now(),
         }
     }
 }
@@ -364,7 +367,6 @@ enum PacketContextInner {
         local_addr: SocketAddr,
         peer_addr: SocketAddr,
         received_wall: WallTime,
-        received: std::time::Instant,
     },
     Dummy,
 }
